@@ -1,6 +1,10 @@
 package com.unciv.app
 
-import android.content.*
+import android.content.BroadcastReceiver
+import android.content.ClipData
+import android.content.ClipboardManager
+import android.content.Context
+import android.content.Intent
 import android.widget.Toast
 import com.badlogic.gdx.backends.android.AndroidApplication
 
@@ -16,4 +20,4 @@ class CopyToClipboardReceiver : BroadcastReceiver() {
         clipboard.setPrimaryClip(clip)
         Toast.makeText(context, context.resources.getString(R.string.Notify_Error_StackTrace_Toast), Toast.LENGTH_SHORT).show()
     }
-}
\ No newline at end of file
+}
