@@ -191,7 +191,7 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
         }
 
         ImageGetter.resetAtlases()
-        ImageGetter.setNewRuleset(ImageGetter.ruleset)  // This needs to come after the settings, since we may have default visual mods
+        ImageGetter.reloadImages()  // This needs to come after the settings, since we may have default visual mods
         val imageGetterTilesets = ImageGetter.getAvailableTilesets()
         val availableTileSets = TileSetCache.getAvailableTilesets(imageGetterTilesets)
         if (settings.tileSet !in availableTileSets) { // If the configured tileset is no longer available, default back
@@ -536,7 +536,7 @@ open class UncivGame(val isConsoleMode: Boolean = false) : Game(), PlatformSpeci
 
     companion object {
         //region AUTOMATICALLY GENERATED VERSION DATA - DO NOT CHANGE THIS REGION, INCLUDING THIS COMMENT
-        val VERSION = Version("4.8.5", 913)
+        val VERSION = Version("4.8.7", 915)
         //endregion
 
         lateinit var Current: UncivGame
