@@ -14,6 +14,7 @@ import com.unciv.logic.map.tile.Tile
 import com.unciv.models.UnitAction
 import com.unciv.models.UnitActionType
 import com.unciv.models.ruleset.Building
+import com.unciv.models.ruleset.unique.LocalUniqueCache
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.Stat
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions
@@ -199,10 +200,12 @@ object SpecificUnitAutomation {
             }
 
             // if we got here, we're pretty close, start looking!
+            val localUniqueCache = LocalUniqueCache()
             val chosenTile = applicableTiles.sortedByDescending {
                 Automation.rankTile(
                     it,
-                    unit.civ
+                    unit.civ,
+                    localUniqueCache
                 )
             }
                 .firstOrNull { unit.movement.canReach(it) }
@@ -500,7 +503,7 @@ object SpecificUnitAutomation {
         val tilesInRange = unit.currentTile.getTilesInDistanceRange(2..unit.getRange())
         var highestTileNukeValue = 0
         var tileToNuke: Tile? = null
-        tilesInRange.forEach { 
+        tilesInRange.forEach {
             val value = getNukeLocationValue(unit, it)
             if (value > highestTileNukeValue) {
                 highestTileNukeValue = value
@@ -524,15 +527,15 @@ object SpecificUnitAutomation {
         val tilesInBlastRadius = tile.getTilesInDistance(blastRadius)
         val civsInBlastRadius = tilesInBlastRadius.mapNotNull { it.getOwner() } +
             tilesInBlastRadius.mapNotNull { it.getFirstUnit()?.civ }
-        
+
         // Don't nuke if it means we will be declaring war on someone!
         if (civsInBlastRadius.any { it != civ && !it.isAtWarWith(civ) }) return -100000
         // If there are no enemies to hit, don't nuke
         if (!civsInBlastRadius.any { it.isAtWarWith(civ) }) return -100000
-        
+
         // Launching a Nuke uses resources, therefore don't launch it by default
         var explosionValue = -500
-        
+
         // Returns either ourValue or thierValue depending on if the input Civ matches the Nuke's Civ
         fun evaluateCivValue(targetCiv: Civilization, ourValue: Int, theirValue: Int): Int {
             if (targetCiv == civ) // We are nuking something that we own!
@@ -548,8 +551,8 @@ object SpecificUnitAutomation {
                     explosionValue += evaluateCivValue(targetTile.civilianUnit?.civ!!, -100, 25)
             }
             // Never nuke our own Civ, don't nuke single enemy civs as well
-            if (targetTile.isCityCenter() 
-                && !(targetTile.getCity()!!.health <= 50f 
+            if (targetTile.isCityCenter()
+                && !(targetTile.getCity()!!.health <= 50f
                     && targetTile.neighbors.any {it.militaryUnit?.civ == civ})) // Prefer not to nuke cities that we are about to take
                 explosionValue += evaluateCivValue(targetTile.getCity()?.civ!!, -100000, 250)
             else if (targetTile.owningCity != null) {
