@@ -3,10 +3,14 @@ package com.unciv.logic.city.managers
 import com.unciv.logic.city.City
 import com.unciv.logic.city.CityFlags
 import com.unciv.logic.city.CityFocus
+import com.unciv.logic.civilization.CityAction
+import com.unciv.logic.civilization.LocationAction
 import com.unciv.logic.civilization.NotificationCategory
 import com.unciv.logic.civilization.NotificationIcon
+import com.unciv.logic.civilization.OverviewAction
 import com.unciv.models.ruleset.tile.ResourceType
 import com.unciv.models.ruleset.unique.UniqueType
+import com.unciv.ui.screens.overviewscreen.EmpireOverviewCategories
 import kotlin.math.min
 import kotlin.random.Random
 
@@ -50,7 +54,7 @@ class CityTurnManager(val city: City) {
             city.setFlag(CityFlags.WeLoveTheKing, 20 + 1) // +1 because it will be decremented by 1 in the same startTurn()
             city.civ.addNotification(
                 "Because they have [${city.demandedResource}], the citizens of [${city.name}] are celebrating We Love The King Day!",
-                city.location, NotificationCategory.General, NotificationIcon.City, NotificationIcon.Happiness)
+                CityAction.withLocation(city), NotificationCategory.General, NotificationIcon.City, NotificationIcon.Happiness)
         }
     }
 
@@ -70,14 +74,14 @@ class CityTurnManager(val city: City) {
                     CityFlags.WeLoveTheKing.name -> {
                         city.civ.addNotification(
                             "We Love The King Day in [${city.name}] has ended.",
-                            city.location, NotificationCategory.General, NotificationIcon.City)
+                            CityAction.withLocation(city), NotificationCategory.General, NotificationIcon.City)
                         demandNewResource()
                     }
                     CityFlags.Resistance.name -> {
                         city.updateCitizens = true
                         city.civ.addNotification(
                             "The resistance in [${city.name}] has ended!",
-                            city.location, NotificationCategory.General, "StatIcons/Resistance")
+                            CityAction.withLocation(city), NotificationCategory.General, "StatIcons/Resistance")
                     }
                 }
             }
@@ -105,7 +109,8 @@ class CityTurnManager(val city: City) {
             city.setFlag(CityFlags.ResourceDemand, 15 + Random.Default.nextInt(10))
         else
             city.civ.addNotification("[${city.name}] demands [${city.demandedResource}]!",
-                city.location, NotificationCategory.General, NotificationIcon.City, "ResourceIcons/${city.demandedResource}")
+                listOf(LocationAction(city.location), OverviewAction(EmpireOverviewCategories.Resources)),
+                NotificationCategory.General, NotificationIcon.City, "ResourceIcons/${city.demandedResource}")
     }
 
 
@@ -144,5 +149,4 @@ class CityTurnManager(val city: City) {
         }
     }
 
-
 }
