@@ -33,6 +33,13 @@ import kotlin.math.ulp
 import kotlin.random.Random
 
 
+/** Map generator, used by new game, map editor and main menu background
+ *
+ *  Class instance only keeps [ruleset] and [coroutineScope] for easier access, input and output are through methods, namely [generateMap] and [generateSingleStep].
+ *
+ *  @param ruleset The Ruleset supplying terrain and resource definitions
+ *  @param coroutineScope Enables early abort if this returns `isActive == false`
+ */
 class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineScope? = null) {
 
     companion object {
@@ -138,23 +145,23 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
         runAndMeasure("RiverGenerator") {
             RiverGenerator(map, randomness, ruleset).spawnRivers()
         }
-        convertTerrains(map, ruleset)
+        convertTerrains(map.values)
 
         // Region based map generation - not used when generating maps in map editor
         if (civilizations.isNotEmpty()) {
             val regions = MapRegions(ruleset)
             runAndMeasure("generateRegions") {
-                regions.generateRegions(map, civilizations.count { ruleset.nations[it.civName]!!.isMajorCiv() })
+                regions.generateRegions(map, civilizations.count { ruleset.nations[it.civName]!!.isMajorCiv })
             }
             runAndMeasure("assignRegions") {
-                regions.assignRegions(map, civilizations.filter { ruleset.nations[it.civName]!!.isMajorCiv() }, gameParameters)
+                regions.assignRegions(map, civilizations.filter { ruleset.nations[it.civName]!!.isMajorCiv }, gameParameters)
             }
             // Natural wonders need to go before most resources since there is a minimum distance
             runAndMeasure("NaturalWonderGenerator") {
                 NaturalWonderGenerator(ruleset, randomness).spawnNaturalWonders(map)
             }
             runAndMeasure("placeResourcesAndMinorCivs") {
-                regions.placeResourcesAndMinorCivs(map, civilizations.filter { ruleset.nations[it.civName]!!.isCityState() })
+                regions.placeResourcesAndMinorCivs(map, civilizations.filter { ruleset.nations[it.civName]!!.isCityState })
             }
         } else {
             runAndMeasure("NaturalWonderGenerator") {
@@ -182,24 +189,27 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
 
         randomness.seedRNG(map.mapParameters.seed)
 
-        when(step) {
-            MapGeneratorSteps.None -> return
-            MapGeneratorSteps.All -> throw IllegalArgumentException("MapGeneratorSteps.All cannot be used in generateSingleStep")
-            MapGeneratorSteps.Landmass -> MapLandmassGenerator(ruleset, randomness).generateLand(map)
-            MapGeneratorSteps.Elevation -> raiseMountainsAndHills(map)
-            MapGeneratorSteps.HumidityAndTemperature -> applyHumidityAndTemperature(map)
-            MapGeneratorSteps.LakesAndCoast -> spawnLakesAndCoasts(map)
-            MapGeneratorSteps.Vegetation -> spawnVegetation(map)
-            MapGeneratorSteps.RareFeatures -> spawnRareFeatures(map)
-            MapGeneratorSteps.Ice -> spawnIce(map)
-            MapGeneratorSteps.Continents -> map.assignContinents(TileMap.AssignContinentsMode.Reassign)
-            MapGeneratorSteps.NaturalWonders -> NaturalWonderGenerator(ruleset, randomness).spawnNaturalWonders(map)
-            MapGeneratorSteps.Rivers -> {
-                RiverGenerator(map, randomness, ruleset).spawnRivers()
-                convertTerrains(map, ruleset)
+        runAndMeasure("SingleStep $step") {
+            when (step) {
+                MapGeneratorSteps.None -> Unit
+                MapGeneratorSteps.All -> throw IllegalArgumentException("MapGeneratorSteps.All cannot be used in generateSingleStep")
+                MapGeneratorSteps.Landmass -> MapLandmassGenerator(ruleset, randomness).generateLand(map)
+                MapGeneratorSteps.Elevation -> raiseMountainsAndHills(map)
+                MapGeneratorSteps.HumidityAndTemperature -> applyHumidityAndTemperature(map)
+                MapGeneratorSteps.LakesAndCoast -> spawnLakesAndCoasts(map)
+                MapGeneratorSteps.Vegetation -> spawnVegetation(map)
+                MapGeneratorSteps.RareFeatures -> spawnRareFeatures(map)
+                MapGeneratorSteps.Ice -> spawnIce(map)
+                MapGeneratorSteps.Continents -> map.assignContinents(TileMap.AssignContinentsMode.Reassign)
+                MapGeneratorSteps.NaturalWonders -> NaturalWonderGenerator(ruleset, randomness).spawnNaturalWonders(map)
+                MapGeneratorSteps.Rivers -> {
+                    val resultingTiles = mutableSetOf<Tile>()
+                    RiverGenerator(map, randomness, ruleset).spawnRivers(resultingTiles)
+                    convertTerrains(resultingTiles)
+                }
+                MapGeneratorSteps.Resources -> spreadResources(map)
+                MapGeneratorSteps.AncientRuins -> spreadAncientRuins(map)
             }
-            MapGeneratorSteps.Resources -> spreadResources(map)
-            MapGeneratorSteps.AncientRuins -> spreadAncientRuins(map)
         }
     }
 
@@ -213,18 +223,19 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
         debug("MapGenerator.%s took %s.%sms", text, delta/1000000L, (delta/10000L).rem(100))
     }
 
-    private fun convertTerrains(map: TileMap, ruleset: Ruleset) {
-        for (tile in map.values) {
+    fun convertTerrains(tiles: Iterable<Tile>) {
+        for (tile in tiles) {
             val conversionUnique =
                 tile.getBaseTerrain().getMatchingUniques(UniqueType.ChangesTerrain)
                     .firstOrNull { tile.isAdjacentTo(it.params[1]) }
                     ?: continue
             val terrain = ruleset.terrains[conversionUnique.params[0]] ?: continue
-            if (!terrain.occursOn.contains(tile.getLastTerrain().name)) continue
 
-            if (terrain.type == TerrainType.TerrainFeature)
+            if (terrain.type == TerrainType.TerrainFeature) {
+                if (!terrain.occursOn.contains(tile.lastTerrain.name)) continue
                 tile.addTerrainFeature(terrain.name)
-            else tile.baseTerrain = terrain.name
+            } else
+                tile.baseTerrain = terrain.name
             tile.setTerrainTransients()
         }
     }
@@ -316,7 +327,7 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
             val suitableTiles = candidateTiles
                     .filterNot { it.baseTerrain == Constants.snow && it.isHill() }
                     .filter { it.resource == null
-                            && resource.terrainsCanBeFoundOn.contains(it.getLastTerrain().name) }
+                            && resource.terrainsCanBeFoundOn.contains(it.lastTerrain.name) }
 
             val locations = randomness.chooseSpreadOutLocations(resourcesPerType, suitableTiles, mapRadius)
 
@@ -335,7 +346,7 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
         val suitableTiles = tileMap.values
                 .filterNot { it.baseTerrain == Constants.snow && it.isHill() }
                 .filter { it.resource == null && it.neighbors.none { neighbor -> neighbor.isNaturalWonder() }
-                        && resourcesOfType.any { r -> r.terrainsCanBeFoundOn.contains(it.getLastTerrain().name) } }
+                        && resourcesOfType.any { r -> r.terrainsCanBeFoundOn.contains(it.lastTerrain.name) } }
         val numberOfResources = tileMap.values.count { it.isLand && !it.isImpassible() } *
                 tileMap.mapParameters.resourceRichness
         val locations = randomness.chooseSpreadOutLocations(numberOfResources.toInt(), suitableTiles, mapRadius)
@@ -344,9 +355,9 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
 
         for (tile in locations) {
             val possibleResources = resourcesOfType
-                    .filter { it.terrainsCanBeFoundOn.contains(tile.getLastTerrain().name) }
+                    .filter { it.terrainsCanBeFoundOn.contains(tile.lastTerrain.name) }
             if (possibleResources.isEmpty()) continue
-            val resourceWithLeastAssignments = possibleResources.minByOrNull { resourceToNumber[it.name]!! }!!
+            val resourceWithLeastAssignments = possibleResources.minByOrNull { resourceToNumber[it.name] }!!
             resourceToNumber.add(resourceWithLeastAssignments.name, 1)
             tile.setTileResource(resourceWithLeastAssignments, rng = randomness.RNG)
         }
@@ -635,13 +646,13 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
         val candidateTerrains = vegetationTerrains.flatMap{ it.occursOn }
         // Checking it.baseTerrain in candidateTerrains to make sure forest does not spawn on desert hill
         for (tile in tileMap.values.asSequence().filter { it.baseTerrain in candidateTerrains
-                && it.getLastTerrain().name in candidateTerrains }) {
+                && it.lastTerrain.name in candidateTerrains }) {
 
             val vegetation = (randomness.getPerlinNoise(tile, vegetationSeed, scale = 3.0, nOctaves = 1) + 1.0) / 2.0
 
             if (vegetation <= tileMap.mapParameters.vegetationRichness) {
                 val possibleVegetation = vegetationTerrains.filter { vegetationTerrain ->
-                    vegetationTerrain.occursOn.contains(tile.getLastTerrain().name)
+                    vegetationTerrain.occursOn.contains(tile.lastTerrain.name)
                             && vegetationTerrain.getMatchingUniques(UniqueType.TileGenerationConditions).none {
                             tile.temperature!! < it.params[0].toDouble() || tile.temperature!! > it.params[1].toDouble()
                                     || tile.humidity!! < it.params[2].toDouble() || tile.humidity!! > it.params[3].toDouble()
@@ -714,7 +725,7 @@ class MapGenerator(val ruleset: Ruleset, private val coroutineScope: CoroutineSc
             val candidates = iceEquivalents
                 .filter {
                     it.matches(temperature, 1.0) &&
-                    tile.getLastTerrain().name in it.terrain.occursOn
+                    tile.lastTerrain.name in it.terrain.occursOn
                 }.map { it.terrain.name }
             when (candidates.size) {
                 1 -> tile.addTerrainFeature(candidates.first())
@@ -964,6 +975,6 @@ class MapGenerationRandomness {
             }
         }
         // unreachable due to last loop iteration always returning and initialDistance >= 1
-        throw Exception()
+        throw Exception("Unreachable code reached!")
     }
 }
