@@ -13,7 +13,6 @@ import com.unciv.models.stats.Stats
 import com.unciv.models.translations.getConditionals
 import com.unciv.models.translations.getPlaceholderParameters
 import com.unciv.models.translations.getPlaceholderText
-import com.unciv.models.translations.removeConditionals
 import kotlin.random.Random
 
 
@@ -21,7 +20,7 @@ class Unique(val text: String, val sourceObjectType: UniqueTarget? = null, val s
     /** This is so the heavy regex-based parsing is only activated once per unique, instead of every time it's called
      *  - for instance, in the city screen, we call every tile unique for every tile, which can lead to ANRs */
     val placeholderText = text.getPlaceholderText()
-    val params = text.removeConditionals().getPlaceholderParameters()
+    val params = text.getPlaceholderParameters()
     val type = UniqueType.values().firstOrNull { it.placeholderText == placeholderText }
 
     val stats: Stats by lazy {
@@ -323,14 +322,20 @@ class LocalUniqueCache(val cache:Boolean = true) {
     fun forCityGetMatchingUniques(
         city: City,
         uniqueType: UniqueType,
-        ignoreConditionals: Boolean = false
+        stateForConditionals: StateForConditionals = StateForConditionals(city.civ, city)
     ): Sequence<Unique> {
-        val stateForConditionals = if (ignoreConditionals) StateForConditionals.IgnoreConditionals
-        else StateForConditionals(city.civ, city)
-        return get(
-            "city-${city.id}-${uniqueType.name}-${ignoreConditionals}",
-            city.getMatchingUniques(uniqueType, stateForConditionals)
-        )
+        // City uniques are a combination of *global civ* uniques plus *city relevant* uniques (see City.getMatchingUniques())
+        // We can cache the civ uniques separately, so if we have several cities using the same cache,
+        //   we can cache the list of *civ uniques* to reuse between cities.
+
+        val citySpecificUniques = get(
+            "city-${city.id}-${uniqueType.name}",
+            city.getLocalMatchingUniques(uniqueType, StateForConditionals.IgnoreConditionals)
+        ).filter { it.conditionalsApply(stateForConditionals) }
+
+        val civUniques = forCivGetMatchingUniques(city.civ, uniqueType, stateForConditionals)
+
+        return citySpecificUniques + civUniques
     }
 
     fun forCivGetMatchingUniques(
@@ -340,12 +345,16 @@ class LocalUniqueCache(val cache:Boolean = true) {
             civ
         )
     ): Sequence<Unique> {
-        val sequence = civ.getMatchingUniques(uniqueType, stateForConditionals)
-        if (!cache) return sequence // So we don't need to toString the stateForConditionals
+        val sequence = civ.getMatchingUniques(uniqueType, StateForConditionals.IgnoreConditionals)
+        // The uniques CACHED are ALL civ uniques, regardless of conditional matching.
+        // The uniques RETURNED are uniques AFTER conditional matching.
+        // This allows reuse of the cached values, between runs with different conditionals -
+        //   for example, iterate on all tiles and get StatPercentForObject uniques relevant for each tile,
+        //   each tile will have different conditional state, but they will all reuse the same list of uniques for the civ
         return get(
-            "civ-${civ.civName}-${uniqueType.name}-${stateForConditionals}",
+            "civ-${civ.civName}-${uniqueType.name}",
             sequence
-        )
+        ).filter { it.conditionalsApply(stateForConditionals) }
     }
 
     /** Get cached results as a sequence */
