@@ -2,13 +2,12 @@ package com.unciv.models.translations
 
 import com.badlogic.gdx.Gdx
 import com.badlogic.gdx.files.FileHandle
-import java.nio.charset.Charset
 import kotlin.collections.set
 
 object TranslationFileReader {
 
     const val percentagesFileLocation = "jsons/translations/completionPercentages.properties"
-    val charset: String = Charset.forName("UTF-8").name()
+    val charset: String = Charsets.UTF_8.name()
 
     fun read(file: FileHandle): LinkedHashMap<String, String> {
         val translations = LinkedHashMap<String, String>()
