@@ -136,6 +136,7 @@ enum class KeyboardBinding(
     BuildWonders(Category.CityScreen, "Buildable Wonders", 'w'),
     BuildNationalWonders(Category.CityScreen, "Buildable National Wonders", 'n'),
     BuildOther(Category.CityScreen, "Other Constructions", 'o'),
+    BuildDisabled(Category.CityScreen, "Disabled Constructions", KeyCharAndCode.ctrl('h')),
     NextCity(Category.CityScreen, Input.Keys.RIGHT),
     PreviousCity(Category.CityScreen, Input.Keys.LEFT),
     ShowStats(Category.CityScreen, 's'),
@@ -157,6 +158,12 @@ enum class KeyboardBinding(
     CultureFocus(Category.CityScreen, "[${Stat.Culture.name}] Focus", KeyCharAndCode.ctrl('c')),
     FaithFocus(Category.CityScreen, "[${Stat.Faith.name}] Focus", KeyCharAndCode.UNKNOWN),
 
+    // CityScreenConstructionMenu (not quite cleanly) reuses RaisePriority/LowerPriority, plus:
+    AddConstructionTop(Category.CityScreenConstructionMenu, "Add to the top of the queue", 't'),
+    AddConstructionAll(Category.CityScreenConstructionMenu, "Add to the queue in all cities", KeyCharAndCode.ctrl('a')),
+    AddConstructionAllTop(Category.CityScreenConstructionMenu, "Add or move to the top in all cities", KeyCharAndCode.ctrl('t')),
+    RemoveConstructionAll(Category.CityScreenConstructionMenu, "Remove from the queue in all cities", KeyCharAndCode.ctrl('r')),
+
     // Popups
     Confirm(Category.Popups, "Confirm Dialog", 'y'),
     Cancel(Category.Popups, "Cancel Dialog", 'n'),
@@ -179,6 +186,7 @@ enum class KeyboardBinding(
             override fun checkConflictsIn() = sequenceOf(WorldScreen)
         },
         CityScreen,
+        CityScreenConstructionMenu, // Maybe someday a category hierarchy?
         Popups
         ;
         val label = unCamelCase(name)
