@@ -1,6 +1,8 @@
 package com.unciv.ui.popups
 
 import com.badlogic.gdx.scenes.scene2d.Stage
+import com.badlogic.gdx.utils.Align
+import com.unciv.ui.components.ColorMarkupLabel
 import com.unciv.ui.components.input.onClick
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.utils.Concurrency
@@ -10,6 +12,8 @@ import kotlinx.coroutines.delay
 /**
  * This is an unobtrusive popup which will close itself after a given amount of time.
  * Default time is two seconds (in milliseconds)
+ *
+ * Note: Supports color markup via [ColorMarkupLabel], using «» instead of Gdx's [].
  */
 class ToastPopup (message: String, stageToShowOn: Stage, val time: Long = 2000) : Popup(stageToShowOn){
 
@@ -20,7 +24,11 @@ class ToastPopup (message: String, stageToShowOn: Stage, val time: Long = 2000)
         setFillParent(false)
         onClick { close() }  // or `touchable = Touchable.disabled` so you can operate what's behind
 
-        addGoodSizedLabel(message)
+        add(ColorMarkupLabel(message).apply {
+            wrap = true
+            setAlignment(Align.center)
+        }).width(stageToShowOn.width / 2)
+
         open()
         //move it to the top so its not in the middle of the screen
         //have to be done after open() because open() centers the popup
