@@ -2,7 +2,9 @@ package com.unciv.ui.screens.overviewscreen
 
 import com.badlogic.gdx.graphics.Color
 import com.unciv.Constants
+import com.unciv.GUI
 import com.unciv.logic.civilization.Civilization
+import com.unciv.logic.civilization.Notification
 import com.unciv.ui.components.TabbedPager
 import com.unciv.ui.components.input.KeyCharAndCode
 import com.unciv.ui.images.ImageGetter
@@ -98,4 +100,15 @@ class EmpireOverviewScreen(
         val scrollY = tab.select(selection) ?: return
         tabbedPager.setPageScrollY(tabbedPager.activePage, scrollY)
     }
+
+    /** Helper to show the world screen with a temporary "one-time" notification */
+    // Here because it's common to notification history, resource finder, and city WLTK demanded resource
+    internal fun showOneTimeNotification(notification: Notification?) {
+        if (notification == null) return  // Convenience - easier than a return@lambda for a caller
+        val worldScreen = GUI.getWorldScreen()
+        worldScreen.notificationsScroll.oneTimeNotification = notification
+        GUI.resetToWorldScreen()
+        notification.resetExecuteRoundRobin()
+        notification.execute(worldScreen)
+    }
 }
