@@ -8,7 +8,6 @@ import com.badlogic.gdx.scenes.scene2d.ui.Cell
 import com.badlogic.gdx.scenes.scene2d.ui.Image
 import com.badlogic.gdx.scenes.scene2d.ui.Table
 import com.badlogic.gdx.utils.Align
-import com.unciv.UncivGame
 import com.unciv.logic.civilization.Civilization
 import com.unciv.models.TutorialTrigger
 import com.unciv.models.UncivSound
@@ -18,23 +17,23 @@ import com.unciv.models.ruleset.PolicyBranch
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.translations.fillPlaceholders
 import com.unciv.models.translations.tr
-import com.unciv.ui.images.ImageGetter
-import com.unciv.ui.popups.ConfirmPopup
-import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.components.BorderedTable
-import com.unciv.ui.screens.basescreen.RecreateOnResize
+import com.unciv.ui.components.ColorMarkupLabel
 import com.unciv.ui.components.extensions.addSeparator
 import com.unciv.ui.components.extensions.center
 import com.unciv.ui.components.extensions.colorFromRGB
 import com.unciv.ui.components.extensions.darken
 import com.unciv.ui.components.extensions.disable
 import com.unciv.ui.components.extensions.enable
-import com.unciv.ui.components.extensions.onClick
-import com.unciv.ui.components.extensions.onDoubleClick
+import com.unciv.ui.components.input.onClick
+import com.unciv.ui.components.input.onDoubleClick
 import com.unciv.ui.components.extensions.pad
 import com.unciv.ui.components.extensions.toGroup
 import com.unciv.ui.components.extensions.toLabel
-import com.unciv.ui.screens.worldscreen.WorldScreen
+import com.unciv.ui.images.ImageGetter
+import com.unciv.ui.popups.ConfirmPopup
+import com.unciv.ui.screens.basescreen.BaseScreen
+import com.unciv.ui.screens.basescreen.RecreateOnResize
 import java.lang.Integer.max
 import kotlin.math.abs
 import kotlin.math.min
@@ -50,19 +49,11 @@ private object PolicyColors {
     val branchAdopted = colorFromRGB(100, 90, 10).darken(0.5f)
 }
 
-fun Policy.isAdopted() : Boolean {
-    val worldScreen = UncivGame.Current.worldScreen ?: return false
-    val viewingCiv = worldScreen.selectedCiv
-    return viewingCiv.policies.isAdopted(this.name)
-}
-
-fun Policy.isPickable() : Boolean {
-    val worldScreen = UncivGame.Current.worldScreen ?: return false
-    val viewingCiv = worldScreen.viewingCiv
-    val selectedCiv = worldScreen.selectedCiv
-    if (!worldScreen.isPlayersTurn
+fun Policy.isPickable(viewingCiv: Civilization, canChangeState: Boolean) : Boolean {
+    if (!viewingCiv.isCurrentPlayer()
+            || !canChangeState
             || viewingCiv.isDefeated()
-            || selectedCiv.policies.isAdopted(this.name)
+            || viewingCiv.policies.isAdopted(this.name)
             || this.policyBranchType == PolicyBranchType.BranchComplete
             || !viewingCiv.policies.isAdoptable(this)
             || !viewingCiv.policies.canAdoptPolicy()
@@ -71,7 +62,7 @@ fun Policy.isPickable() : Boolean {
     return true
 }
 
-class PolicyButton(val policy: Policy, size: Float = 30f) : BorderedTable(
+class PolicyButton(viewingCiv: Civilization, canChangeState: Boolean, val policy: Policy, size: Float = 30f) : BorderedTable(
     path = "PolicyScreen/PolicyButton",
     defaultBgBorder = BaseScreen.skinStrings.roundedEdgeRectangleSmallShape,
     defaultBgShape = BaseScreen.skinStrings.roundedEdgeRectangleSmallShape,
@@ -79,6 +70,9 @@ class PolicyButton(val policy: Policy, size: Float = 30f) : BorderedTable(
 
     val icon = ImageGetter.getImage("PolicyIcons/" + policy.name)
 
+    private val isPickable = policy.isPickable(viewingCiv, canChangeState)
+    private val isAdopted = viewingCiv.policies.isAdopted(policy.name)
+
     var isSelected = false
         set(value) {
             field = value
@@ -86,7 +80,6 @@ class PolicyButton(val policy: Policy, size: Float = 30f) : BorderedTable(
         }
 
     init {
-
         borderSize = 2f
         icon.setSize(size*0.7f, size*0.7f)
 
@@ -102,23 +95,16 @@ class PolicyButton(val policy: Policy, size: Float = 30f) : BorderedTable(
     }
 
     fun onClick(function: () -> Unit): PolicyButton {
-        (this as Actor).onClick(function = {
+        (this as Actor).onClick {
             function()
             updateState()
-        })
+        }
         return this
     }
 
-    fun updateState() {
-
-        val worldScreen = UncivGame.Current.worldScreen ?: return
-        val viewingCiv = worldScreen.selectedCiv
-
-        val isPickable = policy.isPickable()
-        val isAdopted = viewingCiv.policies.isAdopted(policy.name)
+    private fun updateState() {
 
         when {
-
             isSelected && isPickable -> {
                 bgColor = PolicyColors.policySelected
             }
@@ -136,23 +122,20 @@ class PolicyButton(val policy: Policy, size: Float = 30f) : BorderedTable(
                 icon.color.a = 0.2f
                 bgColor = PolicyColors.policyNotPickable
             }
-
         }
-
     }
-
 }
 
 
-class PolicyPickerScreen(val worldScreen: WorldScreen, val viewingCiv: Civilization = worldScreen.selectedCiv)
+class PolicyPickerScreen(val viewingCiv: Civilization, val canChangeState: Boolean)
     : PickerScreen(), RecreateOnResize {
 
     object Sizes {
-        val paddingVertical = 10f
-        val paddingHorizontal = 20f
-        val paddingBetweenHor = 10f
-        val paddingBetweenVer = 20f
-        val iconSize = 50f
+        const val paddingVertical = 10f
+        const val paddingHorizontal = 20f
+        const val paddingBetweenHor = 10f
+        const val paddingBetweenVer = 20f
+        const val iconSize = 50f
     }
 
     private var policyNameToButton = HashMap<String, PolicyButton>()
@@ -180,7 +163,7 @@ class PolicyPickerScreen(val worldScreen: WorldScreen, val viewingCiv: Civilizat
             confirmAction()
         }
 
-        if (!worldScreen.canChangeState)
+        if (!canChangeState)
             rightSideButton.disable()
 
         topTable.row()
@@ -242,7 +225,7 @@ class PolicyPickerScreen(val worldScreen: WorldScreen, val viewingCiv: Civilizat
         val policy = button.policy
 
         rightSideButton.isVisible = !viewingCiv.policies.isAdopted(policy.name)
-        if (!policy.isPickable()) {
+        if (!policy.isPickable(viewingCiv, canChangeState)) {
             rightSideButton.disable()
         } else {
             rightSideButton.enable()
@@ -270,7 +253,7 @@ class PolicyPickerScreen(val worldScreen: WorldScreen, val viewingCiv: Civilizat
         val prefHeight = Sizes.paddingVertical *2 + Sizes.iconSize *maxRow + Sizes.paddingBetweenVer *(maxRow - 1)
 
         // Main table
-        val colorBg = if (branch.isAdopted()) PolicyColors.branchAdopted else PolicyColors.branchNotAdopted
+        val colorBg = if (viewingCiv.policies.isAdopted(branch.name)) PolicyColors.branchAdopted else PolicyColors.branchNotAdopted
         val branchGroup = BorderedTable(path="PolicyScreen/PolicyBranchBackground")
             .apply { bgColor = colorBg }
 
@@ -298,7 +281,8 @@ class PolicyPickerScreen(val worldScreen: WorldScreen, val viewingCiv: Civilizat
         val conditionals = LinkedHashMap<UniqueType, ArrayList<String>>()
 
         branch.uniqueMap[UniqueType.OnlyAvailableWhen.text]?.forEach {
-            it.conditionals.forEach {
+            unique ->
+            unique.conditionals.forEach {
                 if (it.type != null) {
                     if (conditionals[it.type] == null)
                         conditionals[it.type] = ArrayList()
@@ -312,8 +296,7 @@ class PolicyPickerScreen(val worldScreen: WorldScreen, val viewingCiv: Civilizat
             for ((k, v) in conditionals) {
                 warning += "• " + k.text.fillPlaceholders(v.joinToString()).tr() + "\n"
             }
-            val warningLabel = warning.toLabel(Color.RED.cpy(), 13)
-            warningLabel.setFillParent(false)
+            val warningLabel = ColorMarkupLabel(warning, Color.RED, fontSize = 13)
             warningLabel.setAlignment(Align.topLeft)
             warningLabel.wrap = true
             labelTable.add(warningLabel).pad(0f, 20f, 17f, 20f).grow()
@@ -538,10 +521,12 @@ class PolicyPickerScreen(val worldScreen: WorldScreen, val viewingCiv: Civilizat
         }.toGroup(15f) else null
         val expandIcon = ImageGetter.getImage("OtherIcons/BackArrow").apply { rotation = 90f }.toGroup(10f)
         table.add(expandIcon).minWidth(15f).expandX().left()
-        table.add(branch.name.tr().uppercase().toLabel(fontSize = 14).apply { setAlignment(Align.center) }).center()
+        table.add(
+            branch.name.tr(hideIcons = true).uppercase().toLabel(fontSize = 14, alignment = Align.center)
+        ).center()
         table.add(icon).expandX().left().padLeft(5f)
 
-        header.setTouchable(Touchable.enabled)
+        header.touchable = Touchable.enabled
 
         header.add(table).minWidth(150f).growX()
         header.pack()
@@ -552,7 +537,7 @@ class PolicyPickerScreen(val worldScreen: WorldScreen, val viewingCiv: Civilizat
     private fun getTopButton(branch: PolicyBranch): Table {
 
         val text: String
-        val isPickable = branch.isPickable()
+        val isPickable = branch.isPickable(viewingCiv, canChangeState)
         var isAdoptedBranch = false
         var percentage = 0f
 
@@ -628,18 +613,13 @@ class PolicyPickerScreen(val worldScreen: WorldScreen, val viewingCiv: Civilizat
         lockIcon.setPosition(table.width, table.height / 2 - lockIcon.height/2)
 
         table.onClick {
-            if (branch.isPickable())
+            if (branch.isPickable(viewingCiv, canChangeState))
                 ConfirmPopup(
                     this,
                     "Are you sure you want to adopt [${branch.name}]?",
                     "Adopt", true, action = {
                         viewingCiv.policies.adopt(branch, false)
-
-                        val policyScreen = PolicyPickerScreen(worldScreen)
-                        policyScreen.scrollPane.scrollPercentX = scrollPane.scrollPercentX
-                        policyScreen.scrollPane.scrollPercentY = scrollPane.scrollPercentY
-                        policyScreen.scrollPane.updateVisualScroll()
-                        game.replaceCurrentScreen(policyScreen)
+                        game.replaceCurrentScreen(recreate())
                     }
                 ).open(force = true)
         }
@@ -647,10 +627,10 @@ class PolicyPickerScreen(val worldScreen: WorldScreen, val viewingCiv: Civilizat
         return table
     }
 
-    private fun getPolicyButton(policy: Policy, size: Float = 30f): PolicyButton {
-        val button = PolicyButton(policy, size = size)
+    private fun getPolicyButton(policy: Policy, size: Float): PolicyButton {
+        val button = PolicyButton(viewingCiv, canChangeState, policy, size = size)
         button.onClick { pickPolicy(button = button) }
-        if (policy.isPickable())
+        if (policy.isPickable(viewingCiv, canChangeState))
             button.onDoubleClick(UncivSound.Policy) { confirmAction() }
         return button
     }
@@ -659,21 +639,21 @@ class PolicyPickerScreen(val worldScreen: WorldScreen, val viewingCiv: Civilizat
         val policy = selectedPolicyButton!!.policy
 
         // Evil people clicking on buttons too fast to confuse the screen - #4977
-        if (!policy.isPickable()) return
+        if (!policy.isPickable(viewingCiv, canChangeState)) return
 
         viewingCiv.policies.adopt(policy)
 
         // If we've moved to another screen in the meantime (great person pick, victory screen) ignore this
-        if (game.screen !is PolicyPickerScreen) {
-            game.popScreen()
-        } else {
-            val policyScreen = PolicyPickerScreen(worldScreen)
-            policyScreen.scrollPane.scrollPercentX = scrollPane.scrollPercentX
-            policyScreen.scrollPane.scrollPercentY = scrollPane.scrollPercentY
-            policyScreen.scrollPane.updateVisualScroll()
-            game.replaceCurrentScreen(policyScreen)  // update policies
-        }
+        // update policies
+        if (game.screen !is PolicyPickerScreen) game.popScreen()
+        else game.replaceCurrentScreen(recreate())
     }
 
-    override fun recreate(): BaseScreen = PolicyPickerScreen(worldScreen, viewingCiv)
+    override fun recreate(): BaseScreen {
+        val newScreen = PolicyPickerScreen(viewingCiv, canChangeState)
+        newScreen.scrollPane.scrollPercentX = scrollPane.scrollPercentX
+        newScreen.scrollPane.scrollPercentY = scrollPane.scrollPercentY
+        newScreen.scrollPane.updateVisualScroll()
+        return newScreen
+    }
 }
