@@ -7,30 +7,32 @@ import com.badlogic.gdx.scenes.scene2d.ui.VerticalGroup
 import com.badlogic.gdx.utils.Align
 import com.unciv.Constants
 import com.unciv.UncivGame
+import com.unciv.logic.VictoryData
 import com.unciv.logic.civilization.Civilization
 import com.unciv.models.metadata.GameSetupInfo
 import com.unciv.models.ruleset.Victory
 import com.unciv.models.translations.tr
-import com.unciv.ui.components.KeyCharAndCode
+import com.unciv.ui.audio.MusicMood
+import com.unciv.ui.audio.MusicTrackChooserFlags
 import com.unciv.ui.components.TabbedPager
 import com.unciv.ui.components.extensions.areSecretKeysPressed
 import com.unciv.ui.components.extensions.enable
-import com.unciv.ui.components.extensions.onClick
 import com.unciv.ui.components.extensions.toLabel
+import com.unciv.ui.components.input.KeyCharAndCode
+import com.unciv.ui.components.input.onClick
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.ui.screens.basescreen.RecreateOnResize
 import com.unciv.ui.screens.newgamescreen.NewGameScreen
 import com.unciv.ui.screens.pickerscreens.PickerScreen
 import com.unciv.ui.screens.worldscreen.WorldScreen
-
-//TODO someoneHasWon should look at gameInfo.victoryData
+import java.util.EnumSet
 
 class VictoryScreen(
     private val worldScreen: WorldScreen,
     pageNumber: Int = 0
 ) : PickerScreen(), RecreateOnResize {
-
+    private val music get() = UncivGame.Current.musicController
     private val gameInfo = worldScreen.gameInfo
     private val playerCiv = worldScreen.viewingCiv
     private val tabs = TabbedPager(separatorColor = Color.WHITE, shortcutScreen = this)
@@ -41,36 +43,44 @@ class VictoryScreen(
 
     private enum class VictoryTabs(
         val key: Char,
-        val iconName: String = "",
         val caption: String? = null,
         val allowAsSecret: Boolean = false
     ) {
-        OurStatus('O', "StatIcons/Specialist", caption = "Our status") {
-            override fun getContent(worldScreen: WorldScreen) = VictoryScreenOurVictory(worldScreen)
+        OurStatus('O', caption = "Our status") {
+            override fun getContent(parent: VictoryScreen) = VictoryScreenOurVictory(parent.worldScreen)
             override fun isHidden(playerCiv: Civilization) = playerCiv.isSpectator()
         },
-        Global('G', "OtherIcons/Nations", caption = "Global status") {
-            override fun getContent(worldScreen: WorldScreen) = VictoryScreenGlobalVictory(worldScreen)
+        Global('G', caption = "Global status") {
+            override fun getContent(parent: VictoryScreen) = VictoryScreenGlobalVictory(parent.worldScreen)
         },
-        Demographics('D', "CityStateIcons/Cultured", allowAsSecret = true) {
-            override fun getContent(worldScreen: WorldScreen) = VictoryScreenDemographics(worldScreen)
+        Illustration('I', allowAsSecret = true) {
+            override fun getContent(parent: VictoryScreen) = VictoryScreenIllustrations(parent, parent.worldScreen)
+            override fun isHidden(playerCiv: Civilization) = !VictoryScreenIllustrations.enablePage(playerCiv.gameInfo)
+        },
+        Demographics('D', allowAsSecret = true) {
+            override fun getContent(parent: VictoryScreen) = VictoryScreenDemographics(parent.worldScreen)
             override fun isHidden(playerCiv: Civilization) = !UncivGame.Current.settings.useDemographics
         },
-        Rankings('R', "CityStateIcons/Cultured", allowAsSecret = true) {
-            override fun getContent(worldScreen: WorldScreen) = VictoryScreenCivRankings(worldScreen)
+        Rankings('R', allowAsSecret = true) {
+            override fun getContent(parent: VictoryScreen) = VictoryScreenCivRankings(parent.worldScreen)
             override fun isHidden(playerCiv: Civilization) = UncivGame.Current.settings.useDemographics
         },
-        Charts('C', "OtherIcons/Charts") {
-            override fun getContent(worldScreen: WorldScreen) = VictoryScreenCharts(worldScreen)
+        Charts('C') {
+            override fun getContent(parent: VictoryScreen) = VictoryScreenCharts(parent.worldScreen)
             override fun isHidden(playerCiv: Civilization) =
                 !playerCiv.isSpectator() && playerCiv.statsHistory.size < 2
         },
-        Replay('P', "OtherIcons/Load", allowAsSecret = true) {
-            override fun getContent(worldScreen: WorldScreen) = VictoryScreenReplay(worldScreen)
+        Replay('P', allowAsSecret = true) {
+            override fun getContent(parent: VictoryScreen) = VictoryScreenReplay(parent.worldScreen)
             override fun isHidden(playerCiv: Civilization) =
-                !playerCiv.isSpectator() && playerCiv.gameInfo.victoryData == null && playerCiv.isAlive()
+                !playerCiv.isSpectator()
+                        && playerCiv.gameInfo.victoryData == null
+                        && playerCiv.isAlive()
+                        // We show the replay after 5 turns. This is to ensure that the replay
+                        // slider doesn't look weird.
+                        && playerCiv.gameInfo.turns < 5
         };
-        abstract fun getContent(worldScreen: WorldScreen): Table
+        abstract fun getContent(parent: VictoryScreen): Table
         open fun isHidden(playerCiv: Civilization) = false
     }
 
@@ -83,10 +93,10 @@ class VictoryScreen(
             val tabHidden = tab.isHidden(playerCiv)
             if (tabHidden && !(tab.allowAsSecret && Gdx.input.areSecretKeysPressed()))
                 continue
-            val icon = if (tab.iconName.isEmpty()) null else ImageGetter.getImage(tab.iconName)
+            val icon = ImageGetter.getImage("VictoryScreenIcons/${tab.name}")
             tabs.addPage(
                 tab.caption ?: tab.name,
-                tab.getContent(worldScreen),
+                tab.getContent(this),
                 icon, iconSize,
                 scrollAlign = Align.topLeft,
                 shortcutKey = KeyCharAndCode(tab.key),
@@ -96,27 +106,17 @@ class VictoryScreen(
         tabs.selectPage(pageNumber)
 
         //**************** Set up bottom area - buttons and description label ****************
-        rightSideButton.isVisible = false
-
-        var someoneHasWon = false
-
-        val playerVictoryType = playerCiv.victoryManager.getVictoryTypeAchieved()
-        if (playerVictoryType != null) {
-            someoneHasWon = true
-            wonOrLost("You have won a [$playerVictoryType] Victory!", playerVictoryType, true)
-        }
-        for (civ in gameInfo.civilizations.filter { it.isMajorCiv() && it != playerCiv }) {
-            val civVictoryType = civ.victoryManager.getVictoryTypeAchieved()
-            if (civVictoryType != null) {
-                someoneHasWon = true
-                wonOrLost("[${civ.civName}] has won a [$civVictoryType] Victory!", civVictoryType, false)
+        when {
+            gameInfo.victoryData != null ->
+                displayWinner(gameInfo.victoryData!!)
+            playerCiv.isDefeated() -> {
+                displayWonOrLost(Victory().defeatString)
+                music.chooseTrack(playerCiv.civName, MusicMood.Defeat, EnumSet.of(MusicTrackChooserFlags.SuffixMustMatch))
+            }
+            else -> {
+                rightSideButton.isVisible = false
+                setDefaultCloseAction()
             }
-        }
-
-        if (playerCiv.isDefeated()) {
-            wonOrLost("", null, false)
-        } else if (!someoneHasWon) {
-            setDefaultCloseAction()
         }
 
         //**************** Set up floating info panels ****************
@@ -146,18 +146,24 @@ class VictoryScreen(
         }
     }
 
-    private fun wonOrLost(description: String, victoryType: String?, hasWon: Boolean) {
-        val victory = playerCiv.gameInfo.ruleset.victories[victoryType]
+    private fun displayWinner(victoryData: VictoryData) {
+        // We could add `, victoryTurn` to the left side - undecided how to display
+        val (winningCiv, victoryType) = victoryData
+        val victory = gameInfo.ruleset.victories[victoryType]
             ?: Victory()  // This contains our default victory/defeat texts
-        val endGameMessage = when {
-                hasWon -> victory.victoryString
-                else -> victory.defeatString
-            }
+        if (winningCiv == playerCiv.civName) {
+            displayWonOrLost("You have won a [$victoryType] Victory!", victory.victoryString)
+            music.chooseTrack(playerCiv.civName, listOf(MusicMood.Victory, MusicMood.Theme), EnumSet.of(MusicTrackChooserFlags.SuffixMustMatch))
+        } else {
+            displayWonOrLost("[$winningCiv] has won a [$victoryType] Victory!", victory.defeatString)
+            music.chooseTrack(playerCiv.civName, MusicMood.Defeat, EnumSet.of(MusicTrackChooserFlags.SuffixMustMatch))
+        }
+    }
 
-        descriptionLabel.setText(description.tr() + "\n" + endGameMessage.tr())
+    private fun displayWonOrLost(vararg descriptions: String) {
+        descriptionLabel.setText(descriptions.joinToString("\n") { it.tr() })
 
         rightSideButton.setText("Start new game".tr())
-        rightSideButton.isVisible = true
         rightSideButton.enable()
         rightSideButton.onClick {
             val newGameSetupInfo = GameSetupInfo(gameInfo)
