@@ -2,7 +2,7 @@ package com.unciv.logic.civilization
 
 import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.testing.GdxTestRunner
-import com.unciv.uniques.TestGame
+import com.unciv.testing.TestGame
 import org.junit.Assert
 import org.junit.Before
 import org.junit.Test
