@@ -5,6 +5,8 @@ import com.unciv.logic.map.mapunit.UnitTurnManager
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.translations.fillPlaceholders
 import com.unciv.testing.GdxTestRunner
+import com.unciv.testing.TestGame
+import com.unciv.ui.screens.pickerscreens.PromotionTree
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions.getImprovementConstructionActions
 import org.junit.Assert
@@ -27,7 +29,7 @@ class UnitUniquesTests {
         // when
         game.makeHexagonalMap(1)
         val cityState = game.addCiv(cityStateType = "Cultured")
-        val cityStateCapitalTile = game.getTile(Vector2(0f, 0f))
+        val cityStateCapitalTile = game.getTile(Vector2.Zero)
         val cityStateCapital = game.addCity(cityState, cityStateCapitalTile)
 
         val mainCiv = game.addCiv("Gain [90] Influence with a [Great Person] gift to a City-State",
@@ -112,4 +114,70 @@ class UnitUniquesTests {
         // This effectively tests whether the G&K rules have not been tampered with, but won't hurt
         Assert.assertNotEquals("Great General stacked with a Hakkapeliitta should NOT have its normal movement points.", baseMovement, actualMovement)
     }
+
+    @Test
+    fun testCanGetPromotionsWithXP() {
+        val civ = game.addCiv(isPlayer = true)
+
+        val centerTile = game.getTile(Vector2.Zero)
+        val unit = game.addUnit("Scout", civ, centerTile)
+        val tree = PromotionTree(unit)
+
+        Assert.assertFalse(tree.allNodes().any { !it.unreachable && tree.canBuyUpTo(it.promotion) })
+        unit.promotions.XP += 10
+        Assert.assertTrue(tree.allNodes().any { !it.unreachable && tree.canBuyUpTo(it.promotion) })
+    }
+
+    @Test
+    fun testPromotionTreeSetUp() {
+        val civ = game.addCiv(isPlayer = true)
+
+        //Creating the promotions
+        val promotionBranch1 = game.createUnitPromotion()
+        promotionBranch1.unitTypes = listOf("Scout")
+
+        val promotionBranch2 = game.createUnitPromotion()
+        promotionBranch2.unitTypes = listOf("Scout")
+
+        val promotionTestBranchA = game.createUnitPromotion()
+        promotionTestBranchA.unitTypes = listOf("Scout")
+        // intentional lists a promotion twice
+        promotionTestBranchA.prerequisites = listOf(promotionBranch1.name, promotionBranch2.name, promotionBranch1.name)
+
+        val promotionTestBranchB = game.createUnitPromotion()
+        promotionTestBranchB.unitTypes = listOf("Scout")
+        // intentional lists a promotion twice
+        promotionTestBranchB.prerequisites = listOf(promotionBranch1.name, promotionTestBranchA.name, promotionBranch2.name, promotionBranch1.name)
+
+        // add unit
+        val centerTile = game.tileMap[0,0]
+        val unit = game.addUnit("Scout", civ, centerTile)
+        var tree = PromotionTree(unit)
+        Assert.assertFalse("We shouldn't be able to get the promotion without XP",
+            tree.canBuyUpTo(promotionTestBranchB))
+
+        unit.promotions.XP += 30
+        unit.promotions.addPromotion(promotionBranch1.name)
+
+        // The Promotion tree needs to be refreshed to check it after gaining a promotion
+        tree.update()
+
+        Assert.assertTrue("Check if we can buy the Promotion now",
+            tree.canBuyUpTo(promotionTestBranchB))
+
+        // Make sure we only have the prerequisite promotions and that it's only listed once each
+        val promotionNode1 = tree.getNode(promotionTestBranchA)!!
+        Assert.assertEquals(promotionNode1.parents.size, 2)
+        Assert.assertTrue(
+            promotionNode1.parents.any { it.promotion == promotionBranch1 } &&
+            promotionNode1.parents.any { it.promotion == promotionBranch2 }
+        )
+        val promotionNode2 = tree.getNode(promotionTestBranchB)!!
+        Assert.assertEquals(promotionNode2.parents.size, 3)
+        Assert.assertTrue(
+            promotionNode2.parents.any { it.promotion == promotionBranch1 } &&
+            promotionNode2.parents.any { it.promotion == promotionTestBranchA } &&
+            promotionNode2.parents.any { it.promotion == promotionBranch2 }
+        )
+    }
 }
